import { BaseRunner, Config, Logger } from '..';
import { CommandHelper } from '../lib/Classes/CommandHelper';
import * as _ from 'lodash';

export default class Runner extends BaseRunner {
  private Config: Config;
  public prefix: string;
  constructor(...args) {
    // @ts-ignore
    super(...args, {
      Name: 'runCommand',
    });
    this.Config = new Config();
  }

  async handlePrefix(commandHelper) {
    return this.client.redis.getPrefix(commandHelper.Guild.id);
  }

  async checkMaintenance(commandHelper) {
    return this.client.redis.getMaintenance();
  }

  async handlePerms(commandHelper, name) {
    if (await this.client.permissions.get(name).run(commandHelper)) {
      return true;
    }
    if (!!this.client.permissions.get(name).hidden) return 'hidden';
    return false;
  }

  async handleParams(commandHelper) {
    return _.split(commandHelper.Message.content, ' ').splice(1);
  }

  // @ts-ignore
  public async run(msg): Promise<void> {
    if (msg.author.bot) return;
    const commandHelper = new CommandHelper(this.client, msg);
    const prefix = await this.handlePrefix(commandHelper);

    if (!_.startsWith(commandHelper.Message.content, prefix)) return;
    const commandName =
        _.toLower(_.split(commandHelper.Message.cleanContent, ' ').shift()
            .slice(prefix.length));
    const cmd = this.client.commands.get(commandName);
    if (!cmd) return;
    const perm = await this.handlePerms(commandHelper, cmd.permission);
    const params = await this.handleParams(commandHelper);
    await this.checkMaintenance(commandHelper);
    if (perm === true) {
      await cmd.run(commandHelper, params);
    } else if (perm === 'hidden') {
      return;
    } else {
      return commandHelper.sendMessage(`You are missing: \`${cmd.permission}\` permission.`);
    }
  }
}
